<?php

namespace Drupal\etracker\Form;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\etracker\Helper\Constants;

/**
 * Configure eTracker settings for this site.
 */
class EtrackerAdminSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'etracker_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      Constants::ETRACKER_SETTINGS_CONFIG_NAME,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(Constants::ETRACKER_SETTINGS_CONFIG_NAME);

    $form['general'] = [
      '#type' => 'details',
      '#title' => $this->t('General settings'),
      '#open' => TRUE,
    ];

    $form['general']['account_key'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('eTracker Account Key'),
      '#description' => $this->t('You can find your unique eTracker account key on etracker.com &gt; Settings &gt; Account &gt; <a href="https://newapp.etracker.com/#/report/accountSettings/accountKey">Account-Key</a>.<br/><strong>Please note:</strong> This is <em>NOT</em> your Account-ID, but an alphanumeric mixed-case string used as <code>data-secure-code</code> in your tracking snippet.'),
      '#default_value' => $config->get('account_key'),
    ];

    // Scope.
    $form['general']['etracker_scope_script'] = [
      '#type' => 'radios',
      '#title' => $this->t('Script position'),
      '#options' => [
        Constants::ETRACKER_SCOPE_SCRIPT_HEADER => $this->t('Header') . ' (' . $this->t('recommended') . ')',
        Constants::ETRACKER_SCOPE_SCRIPT_FOOTER => $this->t('Footer'),
      ],
      '#default_value' => $config->get('etracker_scope_script') ?? Constants::ETRACKER_SCOPE_SCRIPT_HEADER,
      '#description' => $this->t('The JavaScript is built in a way that it does <strong>not</strong> block or delay the loading of the page.<br/>To support all functionality of eTracker analytics it is required to have the tracking code in the <code>&lt;head&gt;</code> element of the page. Only change this if really needed.'),
    ];

    // Script settings.
    $script_settings = $config->get('etracker_script_settings');
    $form['script_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Script settings'),
      '#open' => TRUE,
    ];

    $form['script_settings']['data_block_cookies'] = [
      '#type' => 'checkbox',
      '#title' => 'Use cookie-less tracking?',
      '#description' => $this->t('Enables cookie-less tracking by setting <code>data-block-cookies="true"</code> in the eTracker tracking code.<br/>For details see <a href="https://www.etracker.com/cookie-less-etracking/">https://www.etracker.com/cookie-less-etracking/</a>'),
      '#default_value' => $script_settings['data_block_cookies'] ?? TRUE,
    ];

    $form['script_settings']['data_respect_dnt'] = [
      '#type' => 'checkbox',
      '#title' => 'Respect Do-Not-Track (DNT) header?',
      '#description' => $this->t('Respects the Do-Not-Track browser header by setting <code>data-respect-dnt="true"</code> in the eTracker tracking code.'),
      '#default_value' => $script_settings['data_respect_dnt'] ?? TRUE,
    ];

    // Default values.
    $form['default_values'] = [
      '#type' => 'details',
      '#title' => $this->t('Default values'),
      '#open' => TRUE,
    ];

    $form['default_values']['etracker_mode_breadcrumb_as_area'] = [
      '#type' => 'radios',
      '#title' => $this->t('Breadcrumb as area'),
      '#default_value' => $config->get('etracker_mode_breadcrumb_as_area') ?? Constants::ETRACKER_MODE_BREADCRUMB_ON_EXCLUDE_HOME,
      '#description' => $this->t('Select if and how you want to utilize your breadcrumb as an hierarchical area for tracking.<br/>If turned on you can select whether the root of your site will be used as the most top area of the hierarchy or not.'),
      '#options' => [
        Constants::ETRACKER_MODE_BREADCRUMB_OFF => $this->t('Off'),
        Constants::ETRACKER_MODE_BREADCRUMB_ON => $this->t('On'),
        Constants::ETRACKER_MODE_BREADCRUMB_ON_EXCLUDE_HOME => $this->t('On, exclude home'),
      ],
    ];

    // Tracking settings.
    // Vertical tabs wrapper.
    $form['tracking_scope'] = [
      '#type' => 'vertical_tabs',
      '#title' => $this->t('Tracking'),
    ];

    // Pages.
    $form['tracking']['pages'] = [
      '#type' => 'details',
      '#title' => $this->t('Pages'),
      '#group' => 'tracking_scope',
    ];

    $form['tracking']['pages']['etracker_track_path_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Tracking specific pages'),
      '#options' => [
        Constants::ETRACKER_TRACK_PATHS_MODE_ALL => $this->t('All pages except those listed'),
        Constants::ETRACKER_TRACK_PATHS_MODE_LISTED => $this->t('Only the listed pages'),
      ],
      '#default_value' => $config->get('etracker_track_path_mode') ?? Constants::ETRACKER_TRACK_PATHS_MODE_ALL,
    ];
    $form['tracking']['pages']['etracker_track_paths'] = [
      '#type' => 'textarea',
      '#title' => $this->t('<span class="element-invisible">Pages</span>'),
      '#default_value' => $config->get('etracker_track_paths') ?? Constants::ETRACKER_TRACK_PATHS,
      '#description' => $this->t("Specify pages by using their paths. Enter one path per line. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.", [
        '%blog' => 'blog',
        '%blog-wildcard' => 'blog/*',
        '%front' => '<front>',
      ]),
    ];

    // Roles.
    $form['tracking']['roles'] = [
      '#type' => 'details',
      '#title' => $this->t('Roles'),
      '#group' => 'tracking_scope',
    ];

    $form['tracking']['roles']['etracker_track_roles_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Tracking specific roles'),
      '#options' => [
        Constants::ETRACKER_TRACK_ROLES_MODE_ALL => $this->t('All roles except those listed'),
        Constants::ETRACKER_TRACK_ROLES_MODE_LISTED => $this->t('Only the selected roles'),
      ],
      '#default_value' => $config->get('etracker_track_roles_mode') ?? Constants::ETRACKER_TRACK_ROLES_MODE_ALL,
    ];

    $form['tracking']['roles']['etracker_track_roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Roles'),
      '#options' => user_role_names(),
      '#default_value' => $config->get('etracker_track_roles'),
    ];

    // User.
    $form['tracking']['user'] = [
      '#type' => 'details',
      '#title' => $this->t('User'),
      '#group' => 'tracking_scope',
    ];

    $t_permission = ['%permission' => $this->t('opt-in or out of etracker tracking')];
    $form['tracking']['user']['etracker_track_user'] = [
      '#type' => 'radios',
      '#title' => $this->t('Allow users to customize tracking on their account page'),
      '#options' => [
        Constants::ETRACKER_TRACK_USER_NO_CUSTOMIZATION => $this->t('No customization allowed'),
        Constants::ETRACKER_TRACK_USER_TRACKING_ON => $this->t('Tracking on by default, users with %permission permission can opt out',
          $t_permission),
        Constants::ETRACKER_TRACK_USER_TRACKING_OFF => $this->t('Tracking off by default, users with %permission permission can opt in',
          $t_permission),
      ],
      '#default_value' => $config->get('etracker_track_user') ?? Constants::ETRACKER_TRACK_USER_NO_CUSTOMIZATION,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $config = $this->configFactory->getEditable(Constants::ETRACKER_SETTINGS_CONFIG_NAME);

    $config->set('account_key', $form_state->getValue('account_key'));
    $config->set('etracker_scope_script', $form_state->getValue('etracker_scope_script'));

    // Pages.
    $config->set('etracker_track_path_mode', $form_state->getValue('etracker_track_path_mode'));
    $config->set('etracker_track_paths', $form_state->getValue('etracker_track_paths'));

    // Roles.
    $config->set('etracker_track_roles_mode', $form_state->getValue('etracker_track_roles_mode'));
    $config->set('etracker_track_roles', $form_state->getValue('etracker_track_roles'));

    // User.
    $config->set('etracker_track_user', $form_state->getValue('etracker_track_user'));

    // Default Values.
    $config->set('etracker_mode_breadcrumb_as_area', $form_state->getValue('etracker_mode_breadcrumb_as_area'));

    // Script settings.
    $etracker_script_settings = [
      'data_block_cookies' => $form_state->getValue('data_block_cookies'),
      'data_respect_dnt' => $form_state->getValue('data_respect_dnt'),
    ];
    $config->set('etracker_script_settings', $etracker_script_settings);

    if (
      ($config->get('account_key') !== $form_state->getValue('account_key'))
      || ($config->get('etracker_script_settings') !== $etracker_script_settings)
    ) {
      Cache::invalidateTags(['library_info']);
    }

    $config->save();
  }

}
